@@ -1,10 +1,5 @@
 import asyncio
-import configparser
 import logging
-import os
-import re
-from datetime import datetime
-from datetime import timedelta
 from typing import Any
 from typing import cast
 from typing import Dict
@@ -21,20 +16,31 @@ from durations import Duration
 from pkg_resources import iter_entry_points
 from senor_octopus.types import Event
 from senor_octopus.types import FilterCallable
+from senor_octopus.types import LoggerCallable
 from senor_octopus.types import SinkCallable
 from senor_octopus.types import SourceCallable
 from senor_octopus.types import Stream
 
 
+async def log_events(stream: Stream, flow: str, log: LoggerCallable) -> Stream:
+    async for event in stream:
+        log("%s: %s", flow, event)
+        yield event
+
+
 class Node:
-    def __init__(self, name: str):
-        self.name = name
+    def __init__(self, node_name: str):
+        self.name = node_name
 
         self.next: Set[Union["Filter", "Sink"]] = set()
-        self._logger = logging.getLogger(name)
+        self._logger = logging.getLogger(node_name)
+        self._event_logger = logging.getLogger("senor_octopus.events")
 
     @staticmethod
-    def build(name: str, section: Dict[str, Any]) -> Union["Source", "Filter", "Sink"]:
+    def build(
+        node_name: str,
+        section: Dict[str, Any],
+    ) -> Union["Source", "Filter", "Sink"]:
         # load plugin
         try:
             plugin_name = section.pop("plugin")
@@ -47,28 +53,24 @@ class Node:
         except StopIteration:
             raise Exception(f"Invalid plugin name `{plugin_name}`")
 
-        # all uppercase keys are environment variables
-        for key in list(section):
-            if re.match("[A-Z_]", key):
-                os.environ[key] = section.pop(key)
-
-        flow = section.pop("flow")
+        kwargs = section.copy()
+        flow = kwargs.pop("flow").strip()
         if flow.startswith("->"):
-            return Source(name, plugin, **section)
+            return Source(node_name, plugin, **kwargs)
         if flow.endswith("->"):
-            return Sink(name, plugin, **section)
-        return Filter(name, plugin, **section)
+            return Sink(node_name, plugin, **kwargs)
+        return Filter(node_name, plugin, **kwargs)
 
 
 class Source(Node):
     def __init__(
         self,
-        name: str,
+        node_name: str,
         plugin: SourceCallable,
         schedule: Optional[str] = None,
         **extra_kwargs: Any,
     ):
-        super().__init__(name)
+        super().__init__(node_name)
         self.plugin = plugin
         self.schedule = CronTab(schedule) if schedule else None
         self.extra_kwargs = extra_kwargs
@@ -78,12 +80,17 @@ class Source(Node):
         stream = self.plugin(**self.extra_kwargs)
         async with itertools.tee(stream, n=len(self.next)) as streams:
             for node, stream in zip(self.next, streams):
+                stream = log_events(
+                    stream,
+                    f"{self.name} -> {node.name}",
+                    self._event_logger.debug,
+                )
                 await node.run(stream)
 
 
 class Filter(Node):
-    def __init__(self, name: str, plugin: FilterCallable, **extra_kwargs: Any):
-        super().__init__(name)
+    def __init__(self, node_name: str, plugin: FilterCallable, **extra_kwargs: Any):
+        super().__init__(node_name)
         self.plugin = plugin
         self.extra_kwargs = extra_kwargs
 
@@ -92,36 +99,40 @@ class Filter(Node):
         stream = self.plugin(stream, **self.extra_kwargs)
         async with itertools.tee(stream, n=len(self.next)) as streams:
             for node, stream in zip(self.next, streams):
+                stream = log_events(
+                    stream,
+                    f"{self.name} -> {node.name}",
+                    self._event_logger.debug,
+                )
                 await node.run(stream)
 
 
 class Sink(Node):
     def __init__(
         self,
-        name: str,
+        node_name: str,
         plugin: SinkCallable,
         throttle: Optional[str] = None,
         batch: Optional[str] = None,
         **extra_kwargs: Any,
     ):
-        super().__init__(name)
+        super().__init__(node_name)
         self.plugin = plugin
-        self.throttle = (
-            timedelta(seconds=Duration(throttle).to_seconds()) if throttle else None
-        )
+        self.throttle = Duration(throttle).to_seconds() if throttle else None
         self.batch = Duration(batch).to_seconds() if batch else None
         self.extra_kwargs = extra_kwargs
 
-        self.last_run: Optional[datetime] = None
-        self.last_batch_start: Optional[float] = None
+        self.last_run: Optional[float] = None
         self.queue: asyncio.Queue = asyncio.Queue()
         self.task = asyncio.create_task(self.worker())
 
     async def run(self, stream: Stream) -> None:
+        loop = asyncio.get_running_loop()
+
         if (
             self.last_run is not None
             and self.throttle
-            and datetime.utcnow() - self.last_run < self.throttle
+            and loop.time() - self.last_run <= self.throttle
         ):
             self._logger.info(
                 "Last run was %s, skipping this one due to throttle",
@@ -130,6 +141,7 @@ class Sink(Node):
             return
 
         self._logger.info("Running")
+        stream = self.run_and_update_last_run(stream)
 
         # when in batch mode, send events to queue for worker to process
         if self.batch:
@@ -140,8 +152,16 @@ class Sink(Node):
             self._logger.info("Processing events")
             await self.plugin(stream, **self.extra_kwargs)  # type: ignore
 
-        # TODO: only update if at least 1 event was received
-        self.last_run = datetime.utcnow()
+    async def run_and_update_last_run(self, stream: Stream) -> Stream:
+        loop = asyncio.get_running_loop()
+
+        at_least_one = False
+        async for event in stream:  # pragma: no cover
+            at_least_one = True
+            yield event
+
+        if at_least_one:
+            self.last_run = loop.time()
 
     async def worker(self) -> None:
         if self.batch is None:
@@ -152,17 +172,17 @@ class Sink(Node):
         while not canceled:
             # consume queue into a new stream
             stream: List[Event] = []
-            self.last_batch_start = None
+            batch_start: Optional[float] = None
             while True:
                 now = loop.time()
 
                 timeout: Optional[float]
-                if self.last_batch_start is None:
+                if batch_start is None:
                     # batch hasn't started, we can wait forever on a new event
                     timeout = None
                 else:
                     # wait on new event while the batch lasts
-                    timeout = self.last_batch_start + self.batch - now
+                    timeout = batch_start + self.batch - now
 
                 try:
                     event = await asyncio.wait_for(self.queue.get(), timeout)
@@ -176,9 +196,9 @@ class Sink(Node):
                     break
 
                 # first event in a batch?
-                if self.last_batch_start is None:
+                if batch_start is None:
                     self._logger.info("Received event, starting a new batch")
-                    self.last_batch_start = loop.time()
+                    batch_start = loop.time()
 
                 stream.append(event)
                 self.queue.task_done()
@@ -203,14 +223,16 @@ def connected(config, source, target) -> bool:
     return True
 
 
-def build_dag(config: configparser.RawConfigParser) -> Set[Source]:
-    sections = set(config.sections())
+def build_dag(config: Dict[str, Any]) -> Set[Source]:
+    sections = set(config)
     for section in sections:
         if "flow" not in config[section]:
             raise Exception("Invalid config, missing `flow` key")
 
-    sources = {name for name in sections if config[name]["flow"].startswith("->")}
-    sinks = {name for name in sections if config[name]["flow"].endswith("->")}
+    sources = {
+        name for name in sections if config[name]["flow"].strip().startswith("->")
+    }
+    sinks = {name for name in sections if config[name]["flow"].strip().endswith("->")}
     filters = sections - sources - sinks
 
     dag: Set[Source] = set()
@@ -221,7 +243,7 @@ def build_dag(config: configparser.RawConfigParser) -> Set[Source]:
         if target in seen:
             node = seen[target]
         else:
-            node = Node.build(target, dict(config[target]))
+            node = Node.build(target, config[target])
             seen[target] = node
 
         if source:
