@@ -2,7 +2,6 @@ import logging
 from datetime import datetime
 from datetime import timezone
 from typing import List
-from typing import Union
 
 import stockquotes
 from senor_octopus.types import Stream
@@ -10,11 +9,26 @@ from senor_octopus.types import Stream
 _logger = logging.getLogger(__name__)
 
 
-async def stock(symbols: Union[str, List[str]], prefix: str = "hub.stock") -> Stream:
-    _logger.info("Fetching stock data")
+async def stock(symbols: List[str], prefix: str = "hub.stock") -> Stream:
+    """
+    Fetch stock price form Yahoo! Finance.
 
-    if isinstance(symbols, str):
-        symbols = [symbol.strip() for symbol in symbols.split(",")]
+    This source will periodically retrieve the price of
+    stocks from Yahoo! Finance.
+
+    Parameters
+    ----------
+    symbols
+        List of stock symbols
+    prefix
+        Prefix for events from this source
+
+    Yields
+    ------
+    Event
+        Events with price of stocks
+    """
+    _logger.info("Fetching stock data")
 
     for symbol in symbols:
         ticker = stockquotes.Stock(symbol)
